﻿using System;
using System.Collections.Generic;

namespace HttpExerciser
{
    public class ResponsePacketInfo : PacketInfo
    {
        public enum ResponseCodes { Http200 = 200, Http201 = 201, Http204 = 204, Http400 = 400, Http401 = 401, Http404 = 404, Http405 = 405, Http409 = 409, Http412 = 412, Http500 = 500 }
        private string[] ResponseText = { "OK", "Created", "No Content", "Bad Request", "Unauthorized", "Not Found", "Method Not Allowed", "Conflict", "Precondition Failed", "Internal Server Error" };

        public string PacketResponseCodeText { get; set; }
        public Int32 PacketResponseCode { get; set; }
        public bool IsHeadResponse { get; set; }
        public string PacketContentTypeText
        {
            get
            {
                switch (this.PacketContentType)
                {
                    case PacketContentTypes.Any:
                        break;

                    case PacketContentTypes.Html:
                        return ContentTypeStrings[1];

                    case PacketContentTypes.Xml:
                        return ContentTypeStrings[2];

                    case PacketContentTypes.Atom:
                        return ContentTypeStrings[3];

                    case PacketContentTypes.Json:
                        return ContentTypeStrings[4];
                }

                // Default "any" to ATOM
                return ContentTypeStrings[3];
            }
        }

        public ResponsePacketInfo(PacketTypes packetType, Int32 packetSize, string packetUri) :
            base(packetType, packetSize, packetUri)
        {
        }

        public void AssignResponse(ResponseCodes code)
        {
            switch (code)
            {
                case ResponseCodes.Http200:
                    this.PacketResponseCodeText = this.ResponseText[0];
                    this.PacketResponseCode = (Int32)ResponseCodes.Http200;
                    return;

                case ResponseCodes.Http201:
                    this.PacketResponseCodeText = this.ResponseText[1];
                    this.PacketResponseCode = (Int32)ResponseCodes.Http201;
                    return;

                case ResponseCodes.Http204:
                    this.PacketResponseCodeText = this.ResponseText[2];
                    this.PacketResponseCode = (Int32)ResponseCodes.Http204;
                    return;

                case ResponseCodes.Http401:
                    this.PacketResponseCodeText = this.ResponseText[4];
                    this.PacketResponseCode = (Int32)ResponseCodes.Http401;
                    return;

                case ResponseCodes.Http404:
                    this.PacketResponseCodeText = this.ResponseText[5];
                    this.PacketResponseCode = (Int32)ResponseCodes.Http404;
                    return;

                case ResponseCodes.Http405:
                    this.PacketResponseCodeText = this.ResponseText[6];
                    this.PacketResponseCode = (Int32)ResponseCodes.Http405;
                    return;

                case ResponseCodes.Http409:
                    this.PacketResponseCodeText = this.ResponseText[7];
                    this.PacketResponseCode = (Int32)ResponseCodes.Http409;
                    return;

                case ResponseCodes.Http412:
                    this.PacketResponseCodeText = this.ResponseText[8];
                    this.PacketResponseCode = (Int32)ResponseCodes.Http412;
                    return;

                case ResponseCodes.Http500:
                    this.PacketResponseCodeText = this.ResponseText[9];
                    this.PacketResponseCode = (Int32)ResponseCodes.Http500;
                    return;

                default:
                    break;
            }

            this.PacketResponseCodeText = this.ResponseText[3];
            this.PacketResponseCode = 400;
        }
    }
}
