﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace HttpExerciser
{
    public partial class Window1
    {
        private string CreateFullUri(string rawPath)
        {
            string pathAndQuery = (!String.IsNullOrEmpty(rawPath) ? "/" + rawPath : String.Empty) + "/";
            UriBuilder builder = new UriBuilder("http", "*", Properties.Settings.Default.Port, pathAndQuery);
            return builder.ToString();
        }

        private List<string> CreateServerPrefixList()
        {
            List<string> prefixes = new List<string>();
            if (chkContentNegotiation.IsChecked == true)
            {
                // Hook the default URI
                prefixes.Add(CreateFullUri(Properties.Settings.Default.UriDefault));
            }
            else
            {
                // Add the HTML URI
                if (Properties.Settings.Default.ContentHtml)
                {
                    prefixes.Add(CreateFullUri(Properties.Settings.Default.UriHtml));
                }

                // Add the XML URI
                if (Properties.Settings.Default.ContentXml)
                {
                    prefixes.Add(CreateFullUri(Properties.Settings.Default.UriXml));
                }

                // Add the ATOM URI
                if (Properties.Settings.Default.ContentAtom)
                {
                    prefixes.Add(CreateFullUri(Properties.Settings.Default.UriAtom));
                }

                // Add the JSON URI
                if (Properties.Settings.Default.ContentJson)
                {
                    prefixes.Add(CreateFullUri(Properties.Settings.Default.UriJson));
                }
            }
            return prefixes;
        }

        private List<string> CreateClientPrefixList()
        {
            List<string> prefixes = new List<string>();

            // Negotiated HTML
            string uriFormat = "http://localhost:{0}/{1}";
            prefixes.Add(String.Format(uriFormat, Properties.Settings.Default.Port, String.Empty));

            // Negotiated XML
            prefixes.Add(String.Format(uriFormat, Properties.Settings.Default.Port, String.Empty));

            // Negotiated ATOM
            prefixes.Add(String.Format(uriFormat, Properties.Settings.Default.Port, String.Empty));

            // Negotiated JSON
            prefixes.Add(String.Format(uriFormat, Properties.Settings.Default.Port, String.Empty));

            // URI HTML
            prefixes.Add(String.Format(uriFormat, Properties.Settings.Default.Port, Properties.Settings.Default.UriHtml));

            // URI XML
            prefixes.Add(String.Format(uriFormat, Properties.Settings.Default.Port, Properties.Settings.Default.UriXml));

            // URI ATOM
            prefixes.Add(String.Format(uriFormat, Properties.Settings.Default.Port, Properties.Settings.Default.UriAtom));

            // URI JSON
            prefixes.Add(String.Format(uriFormat, Properties.Settings.Default.Port, Properties.Settings.Default.UriJson));

            return prefixes;
        }

        private void HandleControlState(bool enabled)
        {
            chkContentNegotiation.IsEnabled = enabled;
            chkSecure.IsEnabled = enabled;
            gbContent.IsEnabled = enabled;
            gbVerbs.IsEnabled = enabled;
            cmdEditContent.IsEnabled = enabled;
            cmdEditURIs.IsEnabled = enabled;
            cmdGenClient.IsEnabled = !enabled;
        }
    }
}
