﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel.Web;

namespace UriTemplateTableSample
{
    class Program
    {
        // The base address we'd determine from the first request (which
        // is when we'd create the template table). Since this isn't a
        // web application, we'll simply assign the base URI.
        const string BaseUri = "http://www.acme.com";

        // The address path comes from the request, but we'll also 
        // simply assign that for this example.
        const string ApplicationPath = "";

        // Normally the "context" would be an instance of HttpContext,
        // but since this isn't a Web application, we'll fake it.
        delegate void HandleRequest(object context, UriTemplateMatch template);

        // The UriTemplateTable is created in this manner because
        // it will be created in a very similar manner in the
        // multithreaded IIS environment. This is the pattern 
        // you will see in the remainder of the book where
        // UriTemplateTable is used.
        UriTemplateTable _templateTable = null;
        UriTemplateTable TemplateTable
        {
            get
            {
                if (_templateTable == null)
                {
                    _templateTable = new UriTemplateTable();
                    _templateTable.BaseAddress = new Uri(BaseUri);
                    _templateTable.KeyValuePairs.Add(
                        new KeyValuePair<UriTemplate, object>(
                         new UriTemplate(
                             ApplicationPath + "/Products"),
                             new HandleRequest(HandleAllProducts)));
                    _templateTable.KeyValuePairs.Add(
                        new KeyValuePair<UriTemplate, object>(
                            new UriTemplate(
                                ApplicationPath + "/Products/{family}"),
                                new HandleRequest(HandleFamily)));
                    _templateTable.KeyValuePairs.Add(
                         new KeyValuePair<UriTemplate, object>(
                             new UriTemplate(
                                 ApplicationPath + "/Products/{family}/{model}/{number}"),
                                 new HandleRequest(HandleProduct)));
                }

                return _templateTable;
            }
        }

        static void Main(string[] args)
        {
            Program pgm = new Program();
            pgm.Run();
        }

        void Run()
        {
            // Build a set of test URIs. Normally these come into IIS
            // as separate requests, but we'll simulate that by asking
            // for a match all four times.
            string[] testUriSet = {"http://www.acme.com/Products/Eludium/Q/36",
                                   "http://www.acme.com/Products",
                                   "http://www.acme.com/Products/Eludium/Q",
                                   "http://www.acme.com/Products/Eludium"};
            foreach (string uri in testUriSet)
            {
                UriTemplateMatch match = TemplateTable.MatchSingle(new Uri(uri));
                if (match == null)
                {
                    Console.WriteLine("{0} has no match, not found.", uri);
                }
                else
                {
                    HandleRequest handleRequest = (HandleRequest)match.Data;
                    handleRequest(uri, match);
                }
            }

            Console.Write("\nPress <enter> to exit.");
            Console.ReadLine();
        }

        void HandleAllProducts(object context, UriTemplateMatch template)
        {
            // Return product catalog
            Console.WriteLine("{0} matched all products catalog", context.ToString());
        }

        void HandleFamily(object context, UriTemplateMatch template)
        {
            // Return product family information
            Console.WriteLine("{0} matched the family {1}", context.ToString(), template.BoundVariables["family"]);
        }

        void HandleProduct(object context, UriTemplateMatch template)
        {
            // Return product item information
            Console.WriteLine("{0} matched a specific product:", context.ToString());
            foreach (string key in template.BoundVariables.Keys)
            {
                string match = String.Format(" {0}: {1}", key, template.BoundVariables[key]);
                Console.WriteLine(match);
            }
        }
    }
}
