﻿// [Book name], ISBN: [isbn]
// Kenn Scribner and Scott Seely
// This code supplied without any warranties, express or implied. 
// Code is free to use, public domain. 
// Have a nice day!

using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel.Web;
using PhotoShared;
using System.Web.Security;
using System.Web;
using System.Net;
using System.Security.Principal;
using System.ServiceModel.Activation;

namespace PhotoWeb
{
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Required)]
    public class ImageManager : IImageManager
    {
        [WebGet(UriTemplate = "/SayHi/{name}")]
        public virtual string SayHi(string name)
        {
            return string.Format("Hello, {0}!", name);
        }

        [WebInvoke(
            BodyStyle = WebMessageBodyStyle.Bare,
            Method = "POST",
            UriTemplate = "/Images/")]
        public virtual Guid AddImage(ImageItem item)
        {
            Guid retval = Guid.Empty;
            if (HttpContext.Current.User.Identity.IsAuthenticated)
            {
                System.IO.MemoryStream stream = new System.IO.MemoryStream(item.ImageBytes);
                System.Drawing.Image sentImage = System.Drawing.Bitmap.FromStream(stream);
                stream = new System.IO.MemoryStream();
                sentImage.Save(stream, System.Drawing.Imaging.ImageFormat.Jpeg);
                stream.Flush();
                Image image = new Image()
                {
                    Caption = item.Caption,
                    Description = item.Description,
                    ImageBytes = stream.GetBuffer(),
                    ImageId = Guid.NewGuid(),
                    PublicImage = item.PublicImage,
                    Created = DateTime.Now,
                    Updated = DateTime.Now,
                    UserName = HttpContext.Current.User.Identity.Name
                };
                retval = image.ImageId;
                using (PhotoWebDbDataContext dataContext = new PhotoWebDbDataContext())
                {
                    dataContext.Images.InsertOnSubmit(image);
                    dataContext.SubmitChanges();
                }
            }
            else
            {
                WebOperationContext.Current.OutgoingResponse.StatusCode = System.Net.HttpStatusCode.Unauthorized;
            }
            return retval;
        }

        [WebGet(BodyStyle = WebMessageBodyStyle.Bare,
          UriTemplate = "/Image/{imageId}")]
        public virtual ImageItem GetImage(string imageId)
        {
            ImageItem retval = null;
            Guid imageIdToken = Guid.Empty;
            if (Utility.TryParse(imageId, out imageIdToken))
            {
                using (PhotoWebDbDataContext dataContext = new PhotoWebDbDataContext())
                {
                    IEnumerable<Image> images = from image in dataContext.Images
                                                where
                                                    image.ImageId == imageIdToken
                                                select image;

                    string baseUrl = string.Format("http://{0}{1}/{2}", HttpContext.Current.Request.Url.Authority, HttpContext.Current.Request.ApplicationPath, "ImageHandler.ashx");

                    foreach (var image in images)
                    {
                        retval = Utility.ConvertImage(image);
                        retval.ImageUrl = string.Format("{0}/{1}", baseUrl, image.ImageId.ToString("D"));
                        break;
                    }
                }
            }
            if (retval == null)
            {
                WebOperationContext.Current.OutgoingResponse.StatusCode = System.Net.HttpStatusCode.NotFound;
            }
            else if (!retval.PublicImage &&
                !string.Equals(retval.UserName,
                    HttpContext.Current.User.Identity.Name, StringComparison.OrdinalIgnoreCase))
            {
                WebOperationContext.Current.OutgoingResponse.StatusCode = System.Net.HttpStatusCode.NotFound;
            }
            return retval;
        }

        [WebGet(BodyStyle = WebMessageBodyStyle.Bare,
         UriTemplate = "/Images/{username}")]
        public virtual ImageItem[] GetImagesForUser(string username)
        {
            if (string.Equals("@self", username))
            {
                username = HttpContext.Current.User.Identity.Name;
            }
            List<ImageItem> retval = new List<ImageItem>();
            IEnumerable<Image> images = null;
            using (PhotoWebDbDataContext dataContext = new PhotoWebDbDataContext())
            {
                if (string.Equals(HttpContext.Current.User.Identity.Name, username, StringComparison.Ordinal))
                {
                    images = from image in dataContext.Images
                             where
                                 image.UserName == username
                             select image;
                }
                else
                {
                    images = from image in dataContext.Images
                             where
                                 image.UserName == username &&
                                 image.PublicImage
                             select image;
                }
                string baseUrl = string.Format("http://{0}{1}/{2}", HttpContext.Current.Request.Url.Authority, HttpContext.Current.Request.ApplicationPath, "ImageHandler.ashx");
                
                foreach (var image in images)
                {
                    ImageItem imageItem = Utility.ConvertImage(image);
                    imageItem.ImageUrl = string.Format("{0}/{1}", baseUrl, image.ImageId.ToString("D"));
                    retval.Add(imageItem);
                }
            }
            if (retval.Count == 0)
            {
                if (WebOperationContext.Current != null)
                {
                    //WebOperationContext.Current.OutgoingResponse.StatusCode = System.Net.HttpStatusCode.NotFound;
                }
                else
                {
                    // return SOAP Fault
                }
            }

            return retval.ToArray();
        }

        [WebInvoke(BodyStyle = WebMessageBodyStyle.Bare, Method = "PUT",
           UriTemplate = "/Image/{imageId}")]
        public virtual void UpdateImage(ImageItem item, string imageId)
        {
            Guid imageToken = Guid.Empty;
            bool found = false;
            IPrincipal user = HttpContext.Current.User;
            if (!user.Identity.IsAuthenticated)
            {
                WebOperationContext.Current.OutgoingResponse.StatusCode = HttpStatusCode.Unauthorized;
            }
            else if (Utility.TryParse(imageId, out imageToken))
            {
                using (PhotoWebDbDataContext dataContext = new PhotoWebDbDataContext())
                {
                    var images = from image in dataContext.Images
                                 where
                                     image.ImageId == imageToken &&
                                     image.UserName == user.Identity.Name
                                 select image;
                    foreach (var image in images)
                    {
                        found = true;
                        image.PublicImage = item.PublicImage;
                        image.Description = item.Description;
                        image.Caption = item.Caption;
                        image.Updated = DateTime.Now;
                        if (item.ImageBytes != null)
                        {
                            image.ImageBytes = item.ImageBytes;
                        }
                        dataContext.SubmitChanges();
                        break;
                    }
                }
                if (!found)
                {
                    WebOperationContext.Current.OutgoingResponse.StatusCode = System.Net.HttpStatusCode.NotFound;
                }
            }
        }

        [WebInvoke(BodyStyle = WebMessageBodyStyle.Bare, Method = "DELETE",
            UriTemplate = "/Image/{imageId}")]
        public virtual void DeleteImage(string imageId)
        {
            Guid imageToken = Guid.Empty;
            if (!HttpContext.Current.User.Identity.IsAuthenticated)
            {
                WebOperationContext.Current.OutgoingResponse.StatusCode = HttpStatusCode.Unauthorized;
            }
            else
            {
                if (Utility.TryParse(imageId, out imageToken))
                {
                    using (PhotoWebDbDataContext dataContext = new PhotoWebDbDataContext())
                    {
                        var images = from image in dataContext.Images
                                     where
                                         image.ImageId == imageToken &&
                                         image.UserName == HttpContext.Current.User.Identity.Name
                                     select image;
                        foreach (var image in images)
                        {
                            dataContext.Images.DeleteOnSubmit(image);
                        }
                        dataContext.SubmitChanges();
                    }
                }
            }
        }
    }
}
