﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Json;
using System.Linq;
using System.Net;
using System.Runtime.Serialization;
using System.Text;
using System.Windows;
using System.Windows.Browser;
using System.Windows.Controls;

namespace PhotoWebSL
{
    public partial class Page : UserControl
    {
        public Page()
        {
            InitializeComponent();
            this.btnLoadFriend.Content = "Load Friend";
            this.btnLoadMyImages.Content = "Load My Images";
            this.btnAddImage.Content = "Add Image";
            this.btnDeleteImage.Content = "Delete Image(s)";
            this.chkPublic.Content = "Public Image";
            XmlImageService = new Uri(App.Current.Host.Source, "../ImageManager.svc");
            JsonImageService = new Uri(App.Current.Host.Source, "../JsonImageManager.svc");
        }

        Uri XmlImageService { get; set; }

        Uri JsonImageService{ get; set; }

        ObservableCollection<ImageItem> ImageDocument { get; set; }
        
        private void LoadFriend_Click(object sender, RoutedEventArgs e)
        {
            txtCaption.IsEnabled = false;
            txtDescription.IsEnabled = false;
            chkPublic.IsEnabled = false;
            btnAddImage.IsEnabled = false;
            btnDeleteImage.IsEnabled = false;
            // Not that this is pratical, but we'll load friends one way,
            // our own a different way. This just shows how, nothing more. 
            string imageUri = string.Format("{0}/Images/{1}", XmlImageService,
                txtFriendName.Text);
            WebRequest request = WebRequest.Create(new Uri(imageUri));
            request.BeginGetResponse(ImagesResponse, request);
        }

        void ImagesResponse(IAsyncResult result)
        {
            WebRequest request = (WebRequest)result.AsyncState;
            try
            {
                WebResponse response = request.EndGetResponse(result);
                DataContractSerializer serializer = new DataContractSerializer(typeof(ImageItem[]));
                ImageDocument = new ObservableCollection<ImageItem>();
                foreach (ImageItem item in (ImageItem[])serializer.ReadObject(response.GetResponseStream()))
                {
                    ImageDocument.Add(item);
                }
                response.Close();
                Dispatcher.BeginInvoke(delegate
                {
                    lstImages.DataContext= ImageDocument;
                });
            }
            catch (WebException we)
            {
                HttpWebResponse response = (HttpWebResponse)we.Response;
                Debug.WriteLine(we.ToString());
                response.Close();
            }
        }

        private void LoadMyImages_Click(object sender, RoutedEventArgs e)
        {
            txtCaption.IsEnabled = true;
            txtDescription.IsEnabled = true;
            chkPublic.IsEnabled = true;
            btnAddImage.IsEnabled = true;
            btnDeleteImage.IsEnabled = true;
            try
            {
                // Go through the JSON service to show how to read JSON data
                string imageUri = string.Format("{0}/GetImagesForUser", JsonImageService);
                WebRequest request = WebRequest.Create(new Uri(imageUri));
                request.Method = "POST";
                request.ContentType = "application/json";
                request.BeginGetRequestStream(GetMyImagesRequestStreamCallback, request);
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.ToString());
            }
        }

        void GetMyImagesRequestStreamCallback(IAsyncResult result)
        {
            WebRequest request = (WebRequest)(result.AsyncState);
            Stream stream = request.EndGetRequestStream(result);
            JsonObject json = new JsonObject(new KeyValuePair<string, JsonValue>
                ("username", "@self"));
            json.Save(stream);
            stream.Close();
            request.BeginGetResponse(JsonImagesResponse, request);
        }

        void JsonImagesResponse(IAsyncResult result)
        {
            WebRequest request = (WebRequest)result.AsyncState;
            try
            {
                WebResponse response = request.EndGetResponse(result);
                Stream responseStream = response.GetResponseStream();
                JsonObject value = (JsonObject)JsonObject.Load(responseStream);
                ImageDocument = new ObservableCollection<ImageItem>();
                if (value.Count == 1)
                {
                    JsonArray valueArray = (JsonArray)value.Values.ElementAt(0);

                    var imageItems = from JsonObject obj in valueArray
                                     select new ImageItem()
                                     {
                                         Caption = obj["Caption"],
                                         Description = obj["Description"],
                                         ImageId = obj["ImageId"],
                                         ImageUrl = obj["ImageUrl"],
                                         PublicImage = obj["PublicImage"],
                                         UserName = obj["UserName"]
                                     };
                    foreach (ImageItem item in imageItems)
                    {
                        ImageDocument.Add(item);
                    }
                    Dispatcher.BeginInvoke(delegate
                    {
                        lstImages.DataContext = ImageDocument;
                    });
                }
            }
            catch (WebException we)
            {
                HttpWebResponse response = (HttpWebResponse)we.Response;
                Debug.WriteLine(we.ToString());
                response.Close();
            }
            catch (Exception e)
            {
                Dispatcher.BeginInvoke(delegate
                {
                    MessageBox.Show(e.ToString());
                });
            }
        }

        private void DeleteImages_Click(object sender, RoutedEventArgs e)
        {
            if (grdDetails.DataContext != null)
            {
                ImageItem item = (ImageItem)grdDetails.DataContext;
                HtmlPage.Window.Invoke("DeleteImage", item.ImageId.ToString("D"));

                ImageDocument.RemoveAt(lstImages.SelectedIndex);
                grdDetails.DataContext = null;
            }
        }

        private void AddImages_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Multiselect = false;
            ofd.Filter = "JPEG Files|*.jpeg;*.jpg|PNG Files|*.png|GIF Files|*.gif|All|*.*";
            bool? result = ofd.ShowDialog();
            if (result.HasValue && result.Value)
            {
                ImageItem item = new ImageItem();
                Stream stream = ofd.File.OpenRead();
                item.ImageBytes = new byte[ofd.File.Length];
                stream.Read(item.ImageBytes, 0, item.ImageBytes.Length);
                stream.Close();
                string imageUri = string.Format("{0}/Images/", XmlImageService);
                WebRequest request = WebRequest.Create(new Uri(imageUri));
                request.Method = "POST";
                request.ContentType = "application/xml";
                request.BeginGetRequestStream(GetAddImageRequestStreamCallback, new object[] { request, item });
            }
        }

        void GetAddImageRequestStreamCallback(IAsyncResult result)
        {
            object[] stateVariables = (object[])result.AsyncState;
            WebRequest request = (WebRequest)stateVariables[0];
            ImageItem imageItem = (ImageItem)stateVariables[1];
            try
            {
                Stream stream = request.EndGetRequestStream(result);
                DataContractSerializer dcs = new DataContractSerializer(typeof(ImageItem));
                dcs.WriteObject(stream, imageItem);
                stream.Close();
                request.BeginGetResponse(AddResponse, request);
            }
            catch (WebException we)
            {
                HttpWebResponse response = (HttpWebResponse)we.Response;
                Debug.WriteLine(we.ToString());
                response.Close();
            }
        }

        void AddResponse(IAsyncResult result)
        {
            WebRequest request = (WebRequest)result.AsyncState;
            try
            {
                WebResponse response = request.EndGetResponse(result);
                DataContractSerializer dcs = new DataContractSerializer(typeof(Guid));
                Guid imageId = (Guid)dcs.ReadObject(response.GetResponseStream());
                response.Close();
                string imageUri = string.Format("{0}/Image/{1}", XmlImageService,
                        imageId.ToString("D"));
                
                request= WebRequest.Create(new Uri(imageUri));
                request.BeginGetResponse(GetIndividualImageResponse, request);
            }
            catch (WebException we)
            {
                HttpWebResponse response = (HttpWebResponse)we.Response;
                Debug.WriteLine(we.ToString());
                response.Close();
            }
        }

        private void GetIndividualImageResponse(IAsyncResult result)
        {
            WebRequest request = (WebRequest)result.AsyncState;
            try
            {
                WebResponse response = request.EndGetResponse(result);
                DataContractSerializer dcs = new DataContractSerializer(typeof(ImageItem));
                ImageItem imageItem = (ImageItem)dcs.ReadObject(response.GetResponseStream());
                response.Close();
                Dispatcher.BeginInvoke(delegate
                {
                    ImageDocument.Add(imageItem);
                    lstImages.SelectedIndex = ImageDocument.Count - 1;
                    lstImages.ScrollIntoView(lstImages.SelectedItem);
                });
            }
            catch (WebException we)
            {
                HttpWebResponse response = (HttpWebResponse)we.Response;
                Debug.WriteLine(we.ToString());
                response.Close();
            }
        }

        private void ListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (grdDetails.DataContext != null && chkPublic.IsEnabled)
            {
                string imageUri = string.Format("{0}/UpdateImage", JsonImageService,
                    txtFriendName.Text);
                WebRequest request = WebRequest.Create(new Uri(imageUri));
                request.Method = "POST";
                request.ContentType = "application/json";
                ImageItem imageItem = (ImageItem)grdDetails.DataContext;
                UpdateImageRequest uir = new UpdateImageRequest()
                {
                    ImageId = imageItem.ImageId,
                    Item = imageItem
                };

                request.BeginGetRequestStream(GetUpdateImageRequestStreamCallback, 
                    new object[]{request, uir});
            }
            grdDetails.DataContext = ((ListBox)sender).SelectedItem;             
        }

        void GetUpdateImageRequestStreamCallback(IAsyncResult result)
        {
            try
            {
                object[] stateInfo = (object[])(result.AsyncState);
                WebRequest request = (WebRequest)(stateInfo[0]);
                UpdateImageRequest uir = (UpdateImageRequest)(stateInfo[1]);
                Stream stream = request.EndGetRequestStream(result);
                byte[] buffer = Encoding.UTF8.GetBytes(uir.ToJsonValue.ToString());
                stream.Write(buffer, 0, buffer.Length);
                stream.Close();
                request.BeginGetResponse(UpdateResponse, request);
            }
            catch (Exception ex)
            {
                Dispatcher.BeginInvoke(delegate
                {
                    MessageBox.Show(ex.ToString());
                });
            }
        }

        void UpdateResponse(IAsyncResult result)
        {
            WebRequest request = (WebRequest)result.AsyncState;
            try
            {
                WebResponse response = request.EndGetResponse(result);
                response.Close();
            }
            catch (WebException we)
            {
                HttpWebResponse response = (HttpWebResponse)we.Response;
                Debug.WriteLine(we.ToString());
                response.Close();
            }
        }

    }
}
