﻿// [Book name], ISBN: [isbn]
// Kenn Scribner and Scott Seely
// This code supplied without any warranties, express or implied. 
// Code is free to use, public domain. 
// Have a nice day!

using System.IO;

namespace RESTExtensions
{
    class CacheStream : Stream
    {
        public CacheStream(Stream internalStream)
        {
            Cache = new MemoryStream();
            InternalStream = internalStream;
        }

        private Stream InternalStream { get; set; }

        public MemoryStream Cache { get; set; }

        public override bool CanRead { get { return Cache.CanRead; } }

        public override bool CanSeek { get { return Cache.CanSeek; } }

        public override bool CanWrite { get { return Cache.CanWrite; } }

        public override void Close() { InternalStream.Close(); }

        public override void Flush()
        {
            Cache.Flush();
            InternalStream.Flush();
        }

        public override long Length { get { return Cache.Length; } }

        public override long Position 
        { 
            get { return Cache.Position; }
            set { Cache.Position = value; }
        }

        public override int Read(byte[] buffer, int offset, int count)
        {
            return Cache.Read(buffer, offset, count);
        }

        public override long Seek(long offset, SeekOrigin origin)
        {
            return Cache.Seek(offset, origin);
        }

        public override void SetLength(long value) { Cache.SetLength(value); }

        public override void Write(byte[] buffer, int offset, int count)
        {
            InternalStream.Write(buffer, offset, count);
            Cache.Write(buffer, offset, count);
        }
    }
}
