﻿// [Book name], ISBN: [isbn]
// Kenn Scribner and Scott Seely
// This code supplied without any warranties, express or implied. 
// Code is free to use, public domain. 
// Have a nice day!

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Runtime.Serialization;
using System.Web;

namespace RESTExtensions
{
    public class ProcessHandler : IHttpHandler
    {
        delegate void HandleRequest(HttpContext context, UriTemplateMatch template);
        DataContractSerializer _processInfoSerializer = 
            new DataContractSerializer(typeof(ProcessInfo));
        DataContractSerializer _processInfoBasicSerializer = 
            new DataContractSerializer(typeof(IEnumerable<ProcessInfoBasic>));
        UriTemplateTable _templateTable = null;
        object _tableLock = new object();

        UriTemplateTable TemplateTable
        {
            get
            {
                if (_templateTable == null)
                {
                    lock (_tableLock)
                    {
                        if (_templateTable == null)
                        {
                            HttpRequest request = HttpContext.Current.Request;
                            string fullUrl = request.Url.ToString();
                            string baseUrl = fullUrl.Substring(0, fullUrl.IndexOf(
                                request.ApplicationPath));
                            _templateTable = new UriTemplateTable();
                            _templateTable.BaseAddress = new Uri(baseUrl);
                            _templateTable.KeyValuePairs.Add(
                                new KeyValuePair<UriTemplate, object>(
                                 new UriTemplate(
                                     request.ApplicationPath +
                                     "/Process"), new HandleRequest(OutputProcessList)));
                            _templateTable.KeyValuePairs.Add(
                                new KeyValuePair<UriTemplate, object>(
                                    new UriTemplate(request.ApplicationPath +
                                        "/Process/{ProcessId}"),
                                        new HandleRequest(OutputProcessInfo)));
                        }
                    }
                }
                return _templateTable;
            }
        }

        public bool IsReusable
        {
            get { return true; }
        }

        public void ProcessRequest(HttpContext context)
        {
            UriTemplateMatch match = TemplateTable.MatchSingle(context.Request.Url);
            if (match == null)
            {
                context.Response.StatusCode = (int)HttpStatusCode.NotFound;
            }
            else
            {
                HandleRequest handleRequest = (HandleRequest)match.Data;
                handleRequest(context, match);
            }
            //context.Response.End();
        }


        void OutputProcessList(HttpContext context, UriTemplateMatch template)
        {
            IEnumerable<ProcessInfoBasic> processInfoList =
                from Process process in Process.GetProcesses()
                select new ProcessInfoBasic()
                {
                    Id = process.Id,
                    ProcessName = process.ProcessName,
                    Link = new Uri(string.Format("{0}/{1}",
                      context.Request.Url.ToString(),
                      process.Id))
                };

            // Notify caller that the response resource is in XML.
            context.Response.ContentType = "text/xml";
            if (context.Request.HttpMethod == "GET")
            {
                _processInfoBasicSerializer.WriteObject(
                    context.Response.OutputStream, processInfoList);
            }
            else if (context.Request.HttpMethod == "HEAD")
            {
                context.Response.Headers["Content-Length"] = "201";
            }
        }

        void OutputProcessInfo(HttpContext context, UriTemplateMatch template)
        {
            int processId = 0;
            if (int.TryParse(template.BoundVariables["ProcessId"], out processId))
            {
                try
                {
                    ProcessInfo processInfo = new ProcessInfo(processId);

                    // Notify caller that the response resource is in XML.
                    context.Response.ContentType = "text/xml";
                    _processInfoSerializer.WriteObject(
                        context.Response.OutputStream, processInfo);
                }
                catch (ArgumentException ex)
                {
                    // The message is always appropriate, so just write it out.
                    context.Response.ContentType = "text";
                    context.Response.Write(ex.Message);
                }
            }
            context.Response.Expires = 1;
        }
    }
}
