﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Security.Principal;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using System.Web.UI;
using System.IO;

using AspNetMvcRestService.Models;

namespace AspNetMvcRestService.Controllers
{
    [HandleError]
    [OutputCache(Location = OutputCacheLocation.None)]
    public class CodeXRCController : Controller
    {
        [AcceptVerbs("HEAD")]
        [ActionName("ServiceRequest")]
        public ActionResult HeadRequest(string contentUri)
        {
            // Filter the query string and remove any trailing '/' so
            // we don't have an extra array element.
            contentUri = FilterUri(contentUri);

            // Save the URI parameters
            string[] directives = null;
            if (!String.IsNullOrEmpty(contentUri))
            {
                directives = contentUri.Split('/');
            }

            // Output to a temporary stream
            MemoryStream strm = new MemoryStream();
            this.HttpContext.Items.Add("OutputStream", strm);

            // Handle the request
            CodeXRCGetService result = new CodeXRCGetService(directives); //, true);
            return result;
        }

        [AcceptVerbs("GET")]
        [ActionName("ServiceRequest")]
        public ActionResult GetRequest(string contentUri)
        {
            // Filter the query string and remove any trailing '/' so
            // we don't have an extra array element.
            contentUri = FilterUri(contentUri);

            // Save the URI parameters
            string[] directives = null;
            if (!String.IsNullOrEmpty(contentUri))
            {
                directives = contentUri.Split('/');
            }

            // Output to the true response stream
            this.HttpContext.Items.Add("OutputStream", this.HttpContext.Response.OutputStream);

            // Handle the request
            CodeXRCGetService result = new CodeXRCGetService(directives);
            return result;
        }

        [AcceptVerbs("PUT")]
        [ActionName("ServiceRequest")]
        public ActionResult PutRequest(string contentUri)
        {
            // Filter the query string and remove any trailing '/' so
            // we don't have an extra array element.
            contentUri = FilterUri(contentUri);

            // Save the URI parameters
            string[] directives = null;
            if (!String.IsNullOrEmpty(contentUri))
            {
                directives = contentUri.Split('/');
            }

            // Output to the true response stream
            this.HttpContext.Items.Add("OutputStream", this.HttpContext.Response.OutputStream);

            // Handle the request
            CodeXRCPutService result = new CodeXRCPutService(directives);
            return result;
        }


        [AcceptVerbs("DELETE")]
        [ActionName("ServiceRequest")]
        public ActionResult DeleteRequest(string contentUri)
        {
            // Filter the query string and remove any trailing '/' so
            // we don't have an extra array element.
            contentUri = FilterUri(contentUri);

            // Save the URI parameters
            string[] directives = null;
            if (!String.IsNullOrEmpty(contentUri))
            {
                directives = contentUri.Split('/');
            }

            // Output to the true response stream
            this.HttpContext.Items.Add("OutputStream", this.HttpContext.Response.OutputStream);

            // Handle the request
            CodeXRCDeleteService result = new CodeXRCDeleteService(directives);
            return result;
        }

        private string FilterUri(string contentUri)
        {
            // If there is a trailing '/', remove it
            string newContentUri = null;
            if (!String.IsNullOrEmpty(contentUri) && contentUri[contentUri.Length - 1] == '/')
            {
                newContentUri = contentUri.Substring(0, contentUri.Length - 1);
            }
            else
            {
                // Use original
                newContentUri = contentUri;
            }

            return newContentUri;
        }
    }
}
