﻿// Effective REST Services via .NET, ISBN: 0321613406
// Kenn Scribner and Scott Seely
// This code supplied without any warranties, express or implied. 
// Code is free to use, public domain. 
// Have a nice day!
using System;
using System.Security.Principal;

namespace PhotoWeb
{
    // We could use the GenericPrincipal and GenericIdentity, but I
    // wanted to show how we might implement them, and I wanted to
    // force the AuthenticationType in the IIdentity value to be
    // "Digest".
    [Serializable]
    public class ServiceUser : IPrincipal
    {
        IIdentity _identity = null;
        string[] _roles = new string[0];

        internal ServiceUser(string username, bool isAuthenticated)
        {
            // Store the identity
            _identity = new ServiceIdentity(username, isAuthenticated) as IIdentity;
        }

        public bool IsInRole(string role)
        {
            // Look up role
            foreach (string testRole in _roles)
            {
                // Compare
                if (role.ToLowerInvariant() == testRole.ToLowerInvariant())
                {
                    // Found it...
                    return true;
                }
            }

            // Role not found
            return false;
        }

        public IIdentity Identity
        {
            get {return _identity; }
        }
    }

    [Serializable]
    public class ServiceIdentity : IIdentity
    {
        private const string AuthenticationTypeString = "Digest";
        private string _username = String.Empty;
        private bool _isAuthenticated = false;

        internal ServiceIdentity(string username, bool isAuthenticated)
        {
            _username = username;
            _isAuthenticated = isAuthenticated;
        }

        public string AuthenticationType
        {
            get { return AuthenticationTypeString; }
        }

        public bool IsAuthenticated
        {
            get { return _isAuthenticated; }
        }

        public string Name
        {
            get { return _username; }
        }
    }
}
