﻿// Effective REST Services via .NET, ISBN: 0321613406
// Kenn Scribner and Scott Seely
// This code supplied without any warranties, express or implied. 
// Code is free to use, public domain. 
// Have a nice day!

using System;
using System.Diagnostics;
using System.ServiceModel.Activation;
using System.ServiceModel.Web;
using System.Web.Security;

namespace PhotoWeb
{
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public class UserManagement : IUserManager
    {
        [WebInvoke(BodyStyle = WebMessageBodyStyle.Bare, Method = "PUT",
            UriTemplate = "/CreateUser/{username}?email={emailAddress}")]
        public void CreateUser(string username, string emailAddress, string password)
        {
            string statusDescription = string.Empty;
            if (Membership.FindUsersByName(username).Count > 0)
            {
                statusDescription = "User name exists";
            }
            else if (Membership.FindUsersByEmail(emailAddress).Count > 0)
            {
                statusDescription = "Email address exists";
            }
            if (!string.IsNullOrEmpty(statusDescription))
            {
                WebOperationContext.Current.OutgoingResponse.StatusCode = System.Net.HttpStatusCode.Conflict;
                WebOperationContext.Current.OutgoingResponse.StatusDescription = statusDescription;
            }
            else
            {
                try
                {
                    MembershipCreateStatus status;
                    Membership.CreateUser(username, password, emailAddress, "Who are you?", 
                        username, true, out status);
                    if (status != MembershipCreateStatus.Success)
                    {
                        WebOperationContext.Current.OutgoingResponse.StatusCode = System.Net.HttpStatusCode.BadRequest;
                        WebOperationContext.Current.OutgoingResponse.StatusDescription = status.ToString();
                    }
                }
                catch (Exception ex)
                {
                    Debug.WriteLine(ex.ToString());
                }
            }
        }

    }
}
