﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Transactions;

using CommentSvc.DAL;

namespace CommentSvc.Model
{
    public class CommentModel
    {
        public CommentDTO GetComment(Int32 commentID)
        {
            // Retrieve the comment
            CommentDTO retVal = null;
            try
            {
                CommentDataClassesDataContext context = new CommentDataClassesDataContext();
                var commentQuery = from c in context.Comments where c.CommentID == commentID select c;

                // Create returned item
                if (commentQuery.Count() > 0)
                {
                    retVal = new CommentDTO(commentQuery.First());
                }
            }
            catch
            {
                // Some error...just return an empty
                retVal = new CommentDTO();
            }

            return retVal;
        }

        public CommentDTO[] GetComments()
        {
            // Retrieve all comments
            CommentDTO[] retVal = null;
            try
            {
                CommentDataClassesDataContext context = new CommentDataClassesDataContext();
                var commentQuery = from c in context.Comments select c;

                // Create returned item list
                List<CommentDTO> items = new List<CommentDTO>();
                foreach (Comment comment in commentQuery)
                {
                    CommentDTO item = new CommentDTO(comment);
                    items.Add(item);
                }
                retVal = items.ToArray();
            }
            catch
            {
                // Some error...just return an empty
                retVal = new CommentDTO[0];
            }

            return retVal;
        }

        public void InsertComment(CommentDTO item)
        {
            // Quick check...
            if (item == null) return;

            try
            {
                // Create the data context
                CommentDataClassesDataContext context = new CommentDataClassesDataContext();

                // Create the comment into the comment table
                Comment comment = new Comment();
                comment.CommentText = item.Comment;
                comment.Name = item.Name;
                comment.EMail = item.EMail;

                // Schedule the new comment for insertion
                context.Comments.InsertOnSubmit(comment);

                // Transact the comment into the database
                using (TransactionScope ts = new TransactionScope())
                {
                    // Submit the entry
                    context.SubmitChanges();

                    // Commit the insert
                    ts.Complete();
                }

                // The comment ID is updated on insert, so we'll return
                // the updated object so that we can create the response
                // to the caller.
                item.CommentID = comment.CommentID;
            }
            catch
            {
                // Some error...just return
            }
        }
    }
}
